/*
 * common.c : shared system foundation functions
 *   by uratan! 2008.9.4
 */
#include "common.h"
#include "LPC214x.h"

	/*
	 * functions prototpes are in common.h
	 */

/* =============================================================== */

/*
 * initial PLL for system clock (CCLK)
 */
void cm_sys_pll_init(void)
{
	PLLCON = 1;				/* enable PLL */

#if (Fcco / Fcclk) == 2
	PLLCFG = ((Fcclk / Fosc) - 1) | (0 << 5);
#elif (Fcco / Fcclk) == 4
	PLLCFG = ((Fcclk / Fosc) - 1) | (1 << 5);
#elif (Fcco / Fcclk) == 8
	PLLCFG = ((Fcclk / Fosc) - 1) | (2 << 5);			/* <== */
#elif (Fcco / Fcclk) == 16
	PLLCFG = ((Fcclk / Fosc) - 1) | (3 << 5);
#else
#error invalid PLLCFG
#endif
	PLLFEED = 0xaa;
	PLLFEED = 0x55;

	/*
	 * wait for PLL lock
	 */
	while((PLLSTAT & (1<<10)) == 0)
		;

	PLLCON = 3;				/* enable & connect PLL */
	PLLFEED = 0xaa;
	PLLFEED = 0x55;
}

/*
 * configure 48MHz USB clock from Fosc=12MHz
 *   it takes about 43usec (24MHz+24MHz)
 */
void cm_usb_pll_init(void)
{
	PLL48CON  = 1;			/* enable PLL */
	PLL48CFG  = 0x23;		/* M = 4, P = 2 */
	PLL48FEED = 0xaa;
	PLL48FEED = 0x55;

	/*
	 * wait for PLL lock
	 */
	while((PLL48STAT & (1<<10)) == 0)
		;

	PLL48CON  = 3;			/* enable & connect PLL */
	PLL48FEED = 0xaa;
	PLL48FEED = 0x55;
}

/* =============================================================== */

#if gUDEBUG
/*
 * initialize UART0
 */
void cm_init_serial0(u_long baudrate)
{
	u_long Fdiv;

	PCONP |= PP_PCUART0;			/* turn UART0 on */

	PINSEL0 |= (1<<2) | (1<<0);		/* enable RxD0 and TxD0 */

	U0IER = 0x00;			/* do not use interrupts */

	U0LCR = 0x80;			/* DLAB=1 */
	U0FDR = 0x10;			/* do not use Fractional Divider */
	Fdiv = (Fpclk / 16) / baudrate;		/* calc division from baudrate */
	U0DLM = Fdiv / 256;
	U0DLL = Fdiv % 256;
	U0LCR = 0x03;			/* DLAB=0, 8bits, no-parity, 1stopbit */

	U0FCR = 0x00;			/* do not use FIFO (inhibited by datasheet!) */
}

/*
 * write a char to UART0
 */
void cm_putchar(int ch)
{
	/*
	 * wait for the buffer empty
	 */
	while(!(U0LSR & 0x20))
		;

	U0THR = ch;
	return;
}

/*
 * chech arrival of a char from UART0
 */
int cm_keyin(void)
{
	if(U0LSR & 0x01) {
		return(U0RBR);	/* something has come */
	}

	return(-1);		/* nothing has come */
}

/*
 * wait until some char will come
 */
u_char cm_getchar(void)
{
	int i;

	while(1) {
		i = cm_keyin();
		if(i != -1)
			break;		/* something arrived */
	}

	return(i);
}
#endif /* gUDEBUG */
