/*
 * link.cmd : Linker Script File, from winARM example 'LPC2138-ROM.ld'
 *   by uratan! 2008.9.8
 */

/*
 * Memory Definitions
 */
MEMORY
{
	ROM (rx) : ORIGIN = 0x00000000, LENGTH = 512k - 12k	/* avoid BOOT BLOCK */
	RAM (rw) : ORIGIN = 0x40000000, LENGTH = 32k - 32	/* avoid IAP work */
}
	/*
	 * ROM usage
	 *   --------------------+---------------------------+------------------
	 *     ROM AREA          |    when our program       | when BOOT LOADER
	 *      start_addr  (len)|        is running         |   is running
	 *   --------------------+---------------------------+------------------
	 *    0x0000_0000-   (64)| ROM vector (if used)      |     (unused)
	 *    0x0000_0040-    (x)|       (free)              |     (unused)
	 *    0x0007_d000-  (12k)| BOOT BLOCK codes          | BOOT BLOCK codes
	 *   --------------------+---------------------------+------------------
	 *           note1: LPC214x may have 2 FLASH memory chip, one for
	 *                  user FLASH, one for boot block ROM.
	 *                  some area of user FLASH is hidden on LPC2148. but
	 *                  on other LPC214x, all user FLASH area is usable.
	 *
	 * RAM usage
	 *   --------------------+---------------------------+------------------
	 *     RAM AREA          |    when our program       | when BOOT LOADER
	 *      start_addr  (len)|        is running         |   is running
	 *   --------------------+---------------------------+------------------
	 *    0x4000_0000-   (64)| RAM vector (if remapped)  |     (unused)
	 *    0x4000_0040-  (224)| RealMonitor work (if used)|     (unused)
	 *    0x4000_0120-  (224)|       (free)              |   ISP work
	 *    0x4000_0200-(31968)|       (free)              |     (unused)
	 *    0x4000_7ee0-  (256)|       (free)              |   ISP stack
	 *    0x4000_7fe0-   (32)| IAP work (if use IAP)     |   ISP work
	 *   --------------------+---------------------------+------------------
	 *           note2: IAP uses our stack, use max 128bytes.
	 */

/*
 * Section Definitions
 */
SECTIONS
{
	/* first section is .text which is used for code */
	.text :
	{
		KEEP(*(.vectorg))
		. = ALIGN(4);
		KEEP(*(.init))			/* Startup code from .init-section */
		*(.text)				/* remaining code */
		*(.text.*)
		*(.rodata)				/* read-only data (constants) */
		*(.rodata*)
		. = ALIGN(4);
	} > ROM
	. = ALIGN(4);
	_text_end = . ;

	/* .data section which is used for initialized data */
	.data : AT (_text_end)
	{
		_data = .;
		*(.data)
		*(.data.*)
		. = ALIGN(4);
	} > RAM
	. = ALIGN(4);
	_data_end = . ;

	/* .bss section which is used for uninitialized data */
	.bss (NOLOAD) :
	{
		_bss_start = . ;
		*(.bss)
		*(COMMON)
		. = ALIGN(4);
	} > RAM
	. = ALIGN(4);
	_bss_end = . ;

	.stack ALIGN(256) :
	{
		*(.stack)
		. = ALIGN(4);
	} > RAM

	_end = . ;
}
