/*
 * pdwon.c : setup EINT1 for power-down and wakeup
 *   by uratan! 2008.9.9
 */
#include "common.h"
#include "LPC214x.h"

/*
 * register bitmaps
 */
/* EXTINT: External Interrupt Flag register */
#define EINT0		0x00000001
#define EINT1		0x00000002
#define EINT2		0x00000004
#define EINT3		0x00000008
/* INTWAKE: Interrupt Wakeup register */
#define EXTWAKE0	0x00000001
#define EXTWAKE1	0x00000002
#define EXTWAKE2	0x00000004
#define EXTWAKE3	0x00000008
#define USBWAKE		0x00000020
#define BODWAKE		0x00004000
#define RTCWAKE		0x00008000
/* EXTMODE: External Interrupt Mode register */
#define EINT0_EDGE	0x00000001
#define EINT1_EDGE	0x00000002
#define EINT2_EDGE	0x00000004
#define EINT3_EDGE	0x00000008
/* EXTPOLAR: External Interrupt Polarity register */
#define EINT0_RISING	0x00000001
#define EINT1_RISING	0x00000002
#define EINT2_RISING	0x00000004
#define EINT3_RISING	0x00000008

/* PCON: Power Control register */
#define PC_IDL         (1<<0)
#define PC_PD          (1<<1)
#define PC_xPDBOD      (1<<2)
#define PC_BODPDM      (1<<3)
#define PC_BOGD        (1<<4)
#define PC_BORD        (1<<5)

/*
 * functions
 */
void pdown_init(void);
int power_down(int en_eint);

/*
 *
 */
void pdown_init(void)
{
	PINSEL0 |= (3<<6);			/* set P0.3 as EINT1 */
//	PINSEL0 |= (2<<28);			/* set P0.14 as EINT1 (for test) */

	INTWAKE = 0;				/* disable wake-up function */
	EXTMODE = EINT1_EDGE;		/* EINT1 edge trigger */
	EXTPOLAR = 0;				/* EINT1 is falling edge by default */
		/*
		 * no need to use intr-handler to wake up operation
		 */
	return;
}

/*
 * enter LPC2148 power-down mode
 */
int power_down(int en_eint)
{
	u_long imf;
	VOLATILE u_long t;
	int wakeup_by_eint;

	imf = IntDisable();				/* disable interrupts */

	/*
	 * once clear request here.
	 *   (means ignore wake-up request before here)
	 */
	EXTINT = EINT1;

#if 0
	/*
	 * wait for UART1 job finished (about 1.3msec with -Os, 60MHz)
	 */
	for(t=0; t<5000; t++)
		;
#else /* 0 */
	/*
	 * wait for UART1 job finished (about 1.1msec with -Os, 24MHz+24MHz)
	 *   the uart baudrate goes wrong by power-down. so if uart is sending
	 *   a data just now, we shall wait for it done. only single-char-
	 *   transmitting-time is enough because interrupt is disabled here.
	 */
	for(t=0; t<2000; t++)
		;
#endif /* 0 */

	/*
	 * select wake up source
	 */
	if(en_eint) {
		INTWAKE = USBWAKE | EXTWAKE1;	/* USB or EINT1 can wake up me */
	} else {
		INTWAKE = USBWAKE;				/* only USB can wake up me */
	}

	/*
	 * then try to Power Down
	 *   this system will never enter Power Down mode if:
	 *     1) wake-up request by EINT1 is already exist.
	 *     2) USB bus activity is already recoverd (USB_need_clk=1).
	 *   (in these case, 2 PLLs keep active)
	 */
	PCON = PC_PD;
		/*
		 * I am in power down mode now! ... maybe
		 *  .
		 *  .
		 *  .
		 */
	/*
	 * I have waked up just now
	 */
	wakeup_by_eint = (EXTINT & EINT1) ? 1 : 0;	/* get status */
	INTWAKE = 0;				/* disable wake-up function */

	/*
	 * re-initial both PLL if needed
	 */
	if((PLLSTAT & (1<<10)) == 0) {
		cm_sys_pll_init();
	}
	if((PLL48STAT & (1<<10)) == 0) {
		cm_usb_pll_init();
	}

	/*
	 * recover previous state
	 */
	IntRestore(imf);				/* (may) enable interrupts */

	return(wakeup_by_eint);
}
