/*
 * usb.h : USB protocol related constants
 *   by uratan! 2008.9.22
 */
#ifndef __USB_H__
#define __USB_H__

/*
 * bmRequestType
 */
#define UT_DIR_MASK         0x80
#define   UT_WRITE            0x00
#define   UT_READ             0x80
#define UT_TYPE_MASK        0x60
#define   UT_STANDARD         0x00
#define   UT_CLASS            0x20
#define   UT_VENDOR           0x40
#define   UT_RESERVED         0x60
#define UT_RECIP_MASK       0x1f
#define   UT_TO_DEVICE        0x00
#define   UT_TO_INTERFACE     0x01
#define   UT_TO_ENDPOINT      0x02
#define   UT_TO_OTHER         0x03

/*
 * USB Standard Request Codes
 */
#define UR_GET_STATUS           0x00
#define UR_CLEAR_FEATURE        0x01
#define UR_SET_FEATURE          0x03
#define UR_SET_ADDRESS          0x05
#define UR_GET_DESCRIPTOR       0x06
#define UR_SET_DESCRIPTOR       0x07
#define UR_GET_CONFIGURATION    0x08
#define UR_SET_CONFIGURATION    0x09
#define UR_GET_INTERFACE        0x0a
#define UR_SET_INTERFACE        0x0b
#define UR_SYNC_FRAME           0x0c

/*
 * Device Status Bit Values
 */
#define UDS_SELF_POWERED    0x01
#define UDS_REMOTE_WAKEUP   0x02

/*
 * Endpoint Status Bit Values
 */
#define UES_HALT            0x01

/*
 * USB Standard Feature selectors
 */
#define UF_ENDPOINT_HALT        0
#define UF_REMOTE_WAKEUP        1

/*
 * USB Descriptor Types
 */
#define UDESC_DEVICE              0x01
#define UDESC_CONFIG              0x02
#define UDESC_STRING              0x03
#define UDESC_INTERFACE           0x04
#define UDESC_ENDPOINT            0x05
#define UDESC_DEVICE_QUALIFIER    0x06
#define UDESC_OTHER_SPEED_CONFIG  0x07
#define UDESC_INTERFACE_POWER     0x08

///* USB Device Classes */
//#define USB_DEVICE_CLASS_RESERVED              0x00
//#define USB_DEVICE_CLASS_AUDIO                 0x01
//#define USB_DEVICE_CLASS_COMMUNICATIONS        0x02
//#define USB_DEVICE_CLASS_HUMAN_INTERFACE       0x03
//#define USB_DEVICE_CLASS_MONITOR               0x04
//#define USB_DEVICE_CLASS_PHYSICAL_INTERFACE    0x05
//#define USB_DEVICE_CLASS_POWER                 0x06
//#define USB_DEVICE_CLASS_PRINTER               0x07
//#define USB_DEVICE_CLASS_STORAGE               0x08
//#define USB_DEVICE_CLASS_HUB                   0x09
//#define USB_DEVICE_CLASS_VENDOR_SPECIFIC       0xFF
//
/*
 * bmAttributes in Configuration Descriptor
 */
#define UC_POWERED_MASK      0xC0
#define UC_BUS_POWERED         0x80
#define UC_SELF_POWERED        0x40
#define UC_REMOTE_WAKEUP       0x20

///* USB Standard Device Descriptor */
//typedef struct _USB_DEVICE_DESCRIPTOR {
//	u_char  bLength;
//	u_char  bDescriptorType;
//	u_short bcdUSB;
//	u_char  bDeviceClass;
//	u_char  bDeviceSubClass;
//	u_char  bDeviceProtocol;
//	u_char  bMaxPacketSize0;
//	u_short idVendor;
//	u_short idProduct;
//	u_short bcdDevice;
//	u_char  iManufacturer;
//	u_char  iProduct;
//	u_char  iSerialNumber;
//	u_char  bNumConfigurations;
//} USB_DEVICE_DESCRIPTOR;
//
///* USB Standard Configuration Descriptor */
//typedef struct _USB_CONFIGURATION_DESCRIPTOR {
//	u_char  bLength;
//	u_char  bDescriptorType;
//	u_short wTotalLength;
//	u_char  bNumInterfaces;
//	u_char  bConfigurationValue;
//	u_char  iConfiguration;
//	u_char  bmAttributes;
//	u_char  MaxPower;
//} USB_CONFIGURATION_DESCRIPTOR;
//
///* USB Standard Interface Descriptor */
//typedef struct _USB_INTERFACE_DESCRIPTOR {
//	u_char  bLength;
//	u_char  bDescriptorType;
//	u_char  bInterfaceNumber;
//	u_char  bAlternateSetting;
//	u_char  bNumEndpoints;
//	u_char  bInterfaceClass;
//	u_char  bInterfaceSubClass;
//	u_char  bInterfaceProtocol;
//	u_char  iInterface;
//} USB_INTERFACE_DESCRIPTOR;
//
///* USB Standard Endpoint Descriptor */
//typedef struct _USB_ENDPOINT_DESCRIPTOR {
//	u_char  bLength;
//	u_char  bDescriptorType;
//	u_char  bEndpointAddress;
//	u_char  bmAttributes;
//	u_short wMaxPacketSize;
//	u_char  bInterval;
//} USB_ENDPOINT_DESCRIPTOR;
//
///* USB String Descriptor */
//typedef struct _USB_STRING_DESCRIPTOR {
//	u_char  bLength;
//	u_char  bDescriptorType;
//	u_short bString/*[]*/;
//} USB_STRING_DESCRIPTOR;


#endif  /* __USB_H__ */
